=== modified file 'assess_autoload_credentials.py'
--- assess_autoload_credentials.py	2016-09-23 14:56:42 +0000
+++ assess_autoload_credentials.py	2016-09-27 15:23:51 +0000
@@ -224,7 +224,8 @@
                 bs_manager.client.env.load_yaml()
 
                 bs_manager.client.bootstrap(
-                    upload_tools, bootstrap_series=bs_manager.series,
+                    upload_tools=upload_tools,
+                    bootstrap_series=bs_manager.series,
                     credential=user)
                 bs_manager.client.kill_controller()
 

=== modified file 'assess_bootstrap.py'
--- assess_bootstrap.py	2016-07-01 11:05:05 +0000
+++ assess_bootstrap.py	2016-09-26 19:40:57 +0000
@@ -3,70 +3,45 @@
 
 from argparse import ArgumentParser
 import logging
-import os.path
 import sys
 
 from deploy_stack import (
     BootstrapManager,
     tear_down,
     )
-from jujupy import (
-    client_from_config,
-    )
 from utility import (
+    add_basic_testing_arguments,
     configure_logging,
-    LoggedException,
-    scoped_environ,
-    temp_dir,
-)
+    )
 
 
 log = logging.getLogger("assess_bootstrap")
 
 
-def assess_bootstrap(juju, env, debug, region, temp_env_name):
-    with scoped_environ():
-        juju_bin = os.path.dirname(os.path.abspath(juju))
-        os.environ['PATH'] = '{}:{}'.format(juju_bin, os.environ['PATH'])
-        client = client_from_config(env, juju, debug)
-    jes_enabled = client.is_jes_enabled()
-    if temp_env_name is None:
-        temp_env_name = client.env.environment
-    with temp_dir() as log_dir:
-        bs_manager = BootstrapManager(
-            temp_env_name, client, client, region=region,
-            permanent=jes_enabled, jes_enabled=jes_enabled, log_dir=log_dir,
-            bootstrap_host=None, machines=[], series=None, agent_url=None,
-            agent_stream=None, keep_env=False)
-        with bs_manager.top_context() as machines:
-            with bs_manager.bootstrap_context(machines):
-                tear_down(client, jes_enabled)
-                client.bootstrap()
-            with bs_manager.runtime_context(machines):
-                client.get_status(1)
-                log.info('Environment successfully bootstrapped.')
+def assess_bootstrap(args):
+    bs_manager = BootstrapManager.from_args(args)
+    client = bs_manager.client
+    with bs_manager.top_context() as machines:
+        with bs_manager.bootstrap_context(machines):
+            tear_down(client, client.is_jes_enabled())
+            client.bootstrap()
+        with bs_manager.runtime_context(machines):
+            client.get_status(1)
+            log.info('Environment successfully bootstrapped.')
 
 
 def parse_args(argv=None):
-    parser = ArgumentParser()
-    parser.add_argument('juju', help="The Juju client to use.")
-    parser.add_argument('env', help="The environment to test with.")
-    parser.add_argument('temp_env_name', nargs='?',
-                        help="Temporary environment name to use.")
-    parser.add_argument('--debug', action="store_true", default=False,
-                        help='Use --debug juju logging.')
-    parser.add_argument('--region', help='Override environment region.')
+    parser = ArgumentParser(description='Test the bootstrap command.')
+    add_basic_testing_arguments(parser)
     return parser.parse_args(argv)
 
 
-def main():
-    args = parse_args()
-    configure_logging(logging.INFO)
-    try:
-        assess_bootstrap(**args.__dict__)
-    except LoggedException:
-        sys.exit(1)
+def main(argv=None):
+    args = parse_args(argv)
+    configure_logging(args.verbose)
+    assess_bootstrap(args)
+    return 0
 
 
 if __name__ == '__main__':
-    main()
+    sys.exit(main())

=== modified file 'assess_constraints.py'
--- assess_constraints.py	2016-09-20 20:28:42 +0000
+++ assess_constraints.py	2016-09-26 16:17:29 +0000
@@ -195,12 +195,37 @@
     return data
 
 
+def application_machines(client, application):
+    """Get all the machines used to host the given application."""
+    raw = client.get_juju_output('status', '--format', 'yaml')
+    raw_yaml = yaml.load(raw)
+    try:
+        app_data = raw_yaml['applications'][application]
+        machines = []
+        for (unit, unit_data) in app_data['units'].items():
+            machines.append(unit_data['machine'])
+        return machines
+    except KeyError as error:
+        raise KeyError(error.args, raw_yaml)
+
+
+def prepare_constraint_test(client, constraints, charm_name,
+                            charm_series='xenial'):
+    """Deploy a charm with constraints and data to see if it meets them."""
+    with temp_dir() as charm_dir:
+        deploy_charm_constraint(client, constraints, charm_name,
+                                charm_series, charm_dir)
+        client.wait_for_started()
+        machines = application_machines(client, charm_name)
+        return juju_show_machine_hardware(client, machines[0])
+
+
 def assess_virt_type(client, virt_type):
     """Assess the virt-type option for constraints"""
     if virt_type not in VIRT_TYPES:
         raise JujuAssertionError(virt_type)
     constraints = Constraints(virt_type=virt_type)
-    charm_name = 'virt-type-' + virt_type
+    charm_name = 'virt-type-{}'.format(virt_type)
     charm_series = 'xenial'
     with temp_dir() as charm_dir:
         deploy_charm_constraint(client, constraints, charm_name,
@@ -223,20 +248,22 @@
         VIRT_TYPES.remove("kvm")
 
 
+def get_failure_exception(client, constraints):
+    """Create a JujuAssertionError with a detailed error message."""
+    message = 'Test Failed: on {} with constraints "{}"'.format(
+        client.env.config.get('type'), str(constraints))
+    return JujuAssertionError(message)
+
+
 def assess_instance_type(client, provider, instance_type):
     """Assess the instance-type option for constraints"""
     if instance_type not in INSTANCE_TYPES[provider]:
         raise JujuAssertionError(instance_type)
     constraints = Constraints(instance_type=instance_type)
-    charm_name = 'instance-type-' + instance_type.replace('.', '-')
-    charm_series = 'xenial'
-    with temp_dir() as charm_dir:
-        deploy_charm_constraint(client, constraints, charm_name,
-                                charm_series, charm_dir)
-        client.wait_for_started()
-        data = juju_show_machine_hardware(client, '0')
-        if not constraints.meets_instance_type(data):
-            raise ValueError('Test failed', charm_name)
+    charm_name = 'instance-type-{}'.format(instance_type.replace('.', '-'))
+    data = prepare_constraint_test(client, constraints, charm_name)
+    if not constraints.meets_instance_type(data):
+        raise get_failure_exception(client, constraints)
 
 
 def assess_instance_type_constraints(client, provider=None):
@@ -249,6 +276,36 @@
         assess_instance_type(client, provider, instance_type)
 
 
+def assess_root_disk_constraints(client, values):
+    """Assess deployment with root-disk constraints."""
+    for root_disk in values:
+        constraints = Constraints(root_disk=root_disk)
+        charm_name = 'root-disk-{}'.format(root_disk.lower())
+        data = prepare_constraint_test(client, constraints, charm_name)
+        if not constraints.meets_root_disk(data['root-disk']):
+            raise get_failure_exception(client, constraints)
+
+
+def assess_cores_constraints(client, values):
+    """Assess deployment with cores constraints."""
+    for cores in values:
+        constraints = Constraints(cores=cores)
+        charm_name = 'cores-{}c'.format(cores)
+        data = prepare_constraint_test(client, constraints, charm_name)
+        if not constraints.meets_cores(data['cores']):
+            raise get_failure_exception(client, constraints)
+
+
+def assess_cpu_power_constraints(client, values):
+    """Assess deployment with cpu_power constraints."""
+    for cpu_power in values:
+        constraints = Constraints(cpu_power=cpu_power)
+        charm_name = 'cpu-power-{}cp'.format(cpu_power)
+        data = prepare_constraint_test(client, constraints, charm_name)
+        if not constraints.meets_cpu_power(data['cpu-power']):
+            raise get_failure_exception(client, constraints)
+
+
 def assess_constraints(client, test_kvm=False):
     """Assess deployment with constraints."""
     provider = client.env.config.get('type')
@@ -256,6 +313,9 @@
         assess_virt_type_constraints(client, test_kvm)
     elif 'ec2' == provider:
         assess_instance_type_constraints(client, provider)
+        assess_root_disk_constraints(client, ['16G'])
+        assess_cores_constraints(client, ['2'])
+        assess_cpu_power_constraints(client, ['30'])
 
 
 def parse_args(argv):

=== modified file 'deploy_stack.py'
--- deploy_stack.py	2016-09-20 14:19:15 +0000
+++ deploy_stack.py	2016-09-23 20:41:02 +0000
@@ -836,7 +836,7 @@
                 yield machines + new_machines
 
     @contextmanager
-    def booted_context(self, upload_tools):
+    def booted_context(self, upload_tools, **kwargs):
         """Create a temporary environment in a context manager to run tests in.
 
         Bootstrap a new environment from a temporary config that is suitable
@@ -849,13 +849,17 @@
 
         :param upload_tools: False or True to upload the local agent instead
             of using streams.
+        :param **kwargs: All remaining keyword arguments are passed to the
+        client's bootstrap.
         """
         try:
             with self.top_context() as machines:
                 with self.bootstrap_context(
                         machines, omit_config=self.client.bootstrap_replaces):
                     self.client.bootstrap(
-                        upload_tools, bootstrap_series=self.series)
+                        upload_tools=upload_tools,
+                        bootstrap_series=self.series,
+                        **kwargs)
                 with self.runtime_context(machines):
                     self.client.list_controllers()
                     self.client.list_models()
@@ -866,14 +870,16 @@
             sys.exit(1)
 
     @contextmanager
-    def existing_booted_context(self, upload_tools):
+    def existing_booted_context(self, upload_tools, **kwargs):
         try:
             with self.top_context() as machines:
                 # Existing does less things as there is no pre-cleanup needed.
                 with self.existing_bootstrap_context(
                         machines, omit_config=self.client.bootstrap_replaces):
                     self.client.bootstrap(
-                        upload_tools, bootstrap_series=self.series)
+                        upload_tools=upload_tools,
+                        bootstrap_series=self.series,
+                        **kwargs)
                 with self.runtime_context(machines):
                     yield machines
         except LoggedException:

=== modified file 'jujupy.py'
--- jujupy.py	2016-09-23 15:02:11 +0000
+++ jujupy.py	2016-09-27 15:23:51 +0000
@@ -1158,7 +1158,8 @@
         return '{}/{}'.format(cloud, region)
 
     def get_bootstrap_args(self, upload_tools, config_filename,
-                           bootstrap_series=None, credential=None):
+                           bootstrap_series=None, credential=None,
+                           auto_upgrade=False, metadata_source=None, to=None):
         """Return the bootstrap arguments for the substrate."""
         if self.env.joyent:
             # Only accept kvm packages by requiring >1 cpu core, see lp:1446264
@@ -1174,13 +1175,16 @@
             args.insert(0, '--upload-tools')
         else:
             args.extend(['--agent-version', self.get_matching_agent_version()])
-
         if bootstrap_series is not None:
             args.extend(['--bootstrap-series', bootstrap_series])
-
         if credential is not None:
             args.extend(['--credential', credential])
-
+        if metadata_source is not None:
+            args.extend(['--metadata-source', metadata_source])
+        if auto_upgrade:
+            args.append('--auto-upgrade')
+        if to is not None:
+            args.extend(['--to', to])
         return tuple(args)
 
     def add_model(self, env):
@@ -1245,21 +1249,25 @@
         self.env.user_name = 'admin@local'
 
     def bootstrap(self, upload_tools=False, bootstrap_series=None,
-                  credential=None):
+                  credential=None, auto_upgrade=False, metadata_source=None,
+                  to=None):
         """Bootstrap a controller."""
         self._check_bootstrap()
         with self._bootstrap_config() as config_filename:
             args = self.get_bootstrap_args(
-                upload_tools, config_filename, bootstrap_series, credential)
+                upload_tools, config_filename, bootstrap_series, credential,
+                auto_upgrade, metadata_source, to)
             self.update_user_name()
             self.juju('bootstrap', args, include_e=False)
 
     @contextmanager
-    def bootstrap_async(self, upload_tools=False, bootstrap_series=None):
+    def bootstrap_async(self, upload_tools=False, bootstrap_series=None,
+                        auto_upgrade=False, metadata_source=None, to=None):
         self._check_bootstrap()
         with self._bootstrap_config() as config_filename:
             args = self.get_bootstrap_args(
-                upload_tools, config_filename, bootstrap_series)
+                upload_tools, config_filename, bootstrap_series, None,
+                auto_upgrade, metadata_source, to)
             self.update_user_name()
             with self.juju_async('bootstrap', args, include_e=False):
                 yield
@@ -2309,7 +2317,8 @@
 class EnvJujuClient2B2(EnvJujuClient2B3):
 
     def get_bootstrap_args(self, upload_tools, config_filename,
-                           bootstrap_series=None, credential=None):
+                           bootstrap_series=None, credential=None,
+                           auto_upgrade=False, metadata_source=None, to=None):
         """Return the bootstrap arguments for the substrate."""
         if self.env.joyent:
             # Only accept kvm packages by requiring >1 cpu core, see lp:1446264

=== modified file 'tests/test_assess_bootstrap.py'
--- tests/test_assess_bootstrap.py	2016-06-02 17:25:41 +0000
+++ tests/test_assess_bootstrap.py	2016-09-26 19:26:05 +0000
@@ -13,16 +13,25 @@
 from tests import (
     FakeHomeTestCase,
     TestCase,
-)
+    )
+from utility import (
+    temp_dir,
+    )
 
 
 class TestParseArgs(TestCase):
 
     def test_parse_args(self):
-        args = parse_args(['foo', 'bar'])
-        self.assertEqual(args, Namespace(
-            juju='foo', env='bar', debug=False, region=None,
-            temp_env_name=None))
+        with temp_dir() as log_dir:
+            args = parse_args(['foo', 'bar', log_dir, 'baz'])
+            self.assertEqual(
+                Namespace(
+                    agent_stream=None, agent_url=None, bootstrap_host=None,
+                    deadline=None, debug=False, env='foo', juju_bin='bar',
+                    keep_env=False, logs=log_dir, machine=[], region=None,
+                    series=None, temp_env_name='baz', upload_tools=False,
+                    verbose=20),
+                args)
 
     def test_parse_args_debug(self):
         args = parse_args(['foo', 'bar', '--debug'])
@@ -33,8 +42,8 @@
         self.assertEqual(args.region, 'foo')
 
     def test_parse_args_temp_env_name(self):
-        args = parse_args(['foo', 'bar', 'foo'])
-        self.assertEqual(args.temp_env_name, 'foo')
+        args = parse_args(['fee', 'fi', 'foe', 'fum'])
+        self.assertEqual(args.temp_env_name, 'fum')
 
 
 class TestAssessBootstrap(FakeHomeTestCase):
@@ -62,7 +71,7 @@
                        autospec=True):
                 with patch('deploy_stack.get_machine_dns_name'):
                     with patch('deploy_stack.dump_env_logs_known_hosts'):
-                        assess_bootstrap('/foo', 'bar', False, None, None)
+                        assess_bootstrap(parse_args(['bar', '/foo']))
         self.assertRegexpMatches(
             self.log_stream.getvalue(),
             r"(?m)^INFO Environment successfully bootstrapped.$")
@@ -78,7 +87,10 @@
                        autospec=True):
                 with patch('deploy_stack.get_machine_dns_name'):
                     with patch('deploy_stack.dump_env_logs_known_hosts'):
-                        assess_bootstrap('/foo', 'bar', False, 'baz', 'qux')
+                        args = parse_args(['bar', '/foo'])
+                        args.region = 'baz'
+                        args.temp_env_name = 'qux'
+                        assess_bootstrap(args)
         self.assertRegexpMatches(
             self.log_stream.getvalue(),
             r"(?m)^INFO Environment successfully bootstrapped.$")

=== modified file 'tests/test_assess_constraints.py'
--- tests/test_assess_constraints.py	2016-09-20 21:15:28 +0000
+++ tests/test_assess_constraints.py	2016-09-21 19:50:14 +0000
@@ -7,8 +7,12 @@
 import StringIO
 
 from assess_constraints import (
+    application_machines,
+    assess_cores_constraints,
+    assess_cpu_power_constraints,
+    assess_instance_type_constraints,
+    assess_root_disk_constraints,
     assess_virt_type_constraints,
-    assess_instance_type_constraints,
     Constraints,
     deploy_constraint,
     deploy_charm_constraint,
@@ -87,6 +91,26 @@
         self.assertTrue(meets_min_mem('1G', '1G'))
         self.assertTrue(meets_min_mem('1G', '2G'))
 
+    def test_meets_root_disk(self):
+        constraints = Constraints(root_disk='8G')
+        self.assertTrue(constraints.meets_root_disk('8G'))
+        self.assertFalse(constraints.meets_root_disk('4G'))
+
+    def test_meets_cores(self):
+        constraints = Constraints(cores='2')
+        self.assertTrue(constraints.meets_cores('3'))
+        self.assertFalse(constraints.meets_cores('1'))
+
+    def test_meets_cpu_power(self):
+        constraints = Constraints(cpu_power='20')
+        self.assertTrue(constraints.meets_cpu_power('30'))
+        self.assertFalse(constraints.meets_cpu_power('10'))
+
+    def test_meets_arch(self):
+        constraints = Constraints(arch='amd64')
+        self.assertTrue(constraints.meets_arch('amd64'))
+        self.assertFalse(constraints.meets_arch('i386'))
+
     def test_meets_instance_type(self):
         constraints = Constraints(instance_type='t2.micro')
         data1 = {'mem': '1G', 'cpu-power': '10', 'cores': '1'}
@@ -189,7 +213,9 @@
             with patch('assess_constraints.get_instance_spec', autospec=True,
                        side_effect=mock_get_instance_spec) as spec_mock:
                 with self.patch_hardware([bar_data, baz_data]):
-                    yield spec_mock
+                    with patch('assess_constraints.application_machines',
+                               autospec=True, return_value=['0']):
+                        yield spec_mock
 
     def test_instance_type_constraints(self):
         assert_constraints_calls = ['instance-type=bar', 'instance-type=baz']
@@ -205,7 +231,10 @@
         with self.prepare_deploy_mock() as (fake_client, deploy_mock):
             fake_provider = fake_client.env.config.get('type')
             with self.patch_instance_spec(fake_provider, False):
-                with self.assertRaises(ValueError):
+                with self.assertRaisesRegexp(
+                        JujuAssertionError,
+                        'Test Failed: on {} with constraints "{}"'.format(
+                            fake_provider, 'instance-type=baz')):
                     assess_instance_type_constraints(fake_client)
 
     def test_instance_type_constraints_missing(self):
@@ -214,6 +243,33 @@
             assess_instance_type_constraints(fake_client)
         self.assertFalse(deploy_mock.called)
 
+    def test_root_disk_constraints(self):
+        fake_client = Mock(wraps=fake_juju_client())
+        with patch('assess_constraints.prepare_constraint_test',
+                   autospec=True, return_value={'root-disk': '8G'}
+                   ) as prepare_mock:
+            with self.assertRaises(JujuAssertionError):
+                assess_root_disk_constraints(fake_client, ['8G', '16G'])
+        self.assertEqual(2, prepare_mock.call_count)
+
+    def test_cores_constraints(self):
+        fake_client = Mock(wraps=fake_juju_client())
+        with patch('assess_constraints.prepare_constraint_test',
+                   autospec=True, return_value={'cores': '2'}
+                   ) as prepare_mock:
+            with self.assertRaises(JujuAssertionError):
+                assess_cores_constraints(fake_client, ['2', '4'])
+        self.assertEqual(2, prepare_mock.call_count)
+
+    def test_cpu_power_constraints(self):
+        fake_client = Mock(wraps=fake_juju_client())
+        with patch('assess_constraints.prepare_constraint_test',
+                   autospec=True, return_value={'cpu-power': '20'}
+                   ) as prepare_mock:
+            with self.assertRaises(JujuAssertionError):
+                assess_cpu_power_constraints(fake_client, ['10', '30'])
+        self.assertEqual(2, prepare_mock.call_count)
+
 
 class TestDeploy(TestCase):
 
@@ -250,36 +306,36 @@
 
 class TestJujuWrappers(TestCase):
 
-    SAMPLE_JUJU_SHOW_MACHINE_OUTPUT = """\
-model:
-  name: controller
-  controller: assessconstraints-20160914122952-temp-env
-  cloud: lxd
-  region: localhost
-  version: 2.0-beta18
-machines:
-  "0":
-    juju-status:
-      current: started
-      since: 14 Sep 2016 12:32:17-04:00
-      version: 2.0-beta18
-    dns-name: 10.252.22.39
-    instance-id: juju-7d249e-0
-    machine-status:
-      current: pending
-      since: 14 Sep 2016 12:32:07-04:00
-    series: xenial
-    hardware: arch=amd64 cpu-cores=0 mem=0M
-    controller-member-status: has-vote
-applications: {}"""
+    # Dictionaries showing plausable, but reduced and pre-loaded, output.
+    SAMPLE_SHOW_MACHINE_OUTPUT = {
+        'model': {'name': 'controller'},
+        'machines': {'0': {'hardware': 'arch=amd64 cpu-cores=0 mem=0M'}},
+        'applications': {}
+        }
+
+    SAMPLE_SHOW_MODEL_OUTPUT = {
+        'model': 'UNUSED',
+        'machines': {'0': 'UNUSED', '1': 'UNUSED'},
+        'applications':
+        {'wiki': {'units': {'wiki/0': {'machine': '0'}}},
+         'mysql': {'units': {'mysql/0': {'machine': '1'}}},
+         }}
 
     def test_juju_show_machine_hardware(self):
         """Check the juju_show_machine_hardware data translation."""
-        output_mock = Mock(
-            return_value=self.SAMPLE_JUJU_SHOW_MACHINE_OUTPUT)
+        output_mock = Mock(return_value=self.SAMPLE_SHOW_MACHINE_OUTPUT)
         fake_client = Mock(get_juju_output=output_mock)
-        data = juju_show_machine_hardware(fake_client, '0')
+        with patch('yaml.load', side_effect=lambda x: x):
+            data = juju_show_machine_hardware(fake_client, '0')
         output_mock.assert_called_once_with('show-machine', '0',
                                             '--format', 'yaml')
         self.assertEqual({'arch': 'amd64', 'cpu-cores': '0', 'mem': '0M'},
                          data)
+
+    def test_application_machines(self):
+        output_mock = Mock(return_value=self.SAMPLE_SHOW_MODEL_OUTPUT)
+        fake_client = Mock(get_juju_output=output_mock)
+        with patch('yaml.load', side_effect=lambda x: x):
+            data = application_machines(fake_client, 'wiki')
+        output_mock.assert_called_once_with('status', '--format', 'yaml')
+        self.assertEquals(['0'], data)

=== modified file 'tests/test_deploy_stack.py'
--- tests/test_deploy_stack.py	2016-09-20 14:19:15 +0000
+++ tests/test_deploy_stack.py	2016-09-23 21:19:33 +0000
@@ -1,11 +1,11 @@
 from argparse import (
     Namespace,
-)
+    )
 from contextlib import contextmanager
 from datetime import (
     datetime,
     timedelta,
-)
+    )
 import json
 import logging
 import os
@@ -13,13 +13,13 @@
 import sys
 from unittest import (
     skipIf,
-)
+    )
 
 from mock import (
     call,
     MagicMock,
     patch,
-)
+    )
 import yaml
 
 from deploy_stack import (
@@ -45,11 +45,11 @@
     safe_print_status,
     retain_config,
     update_env,
-)
+    )
 from fakejuju import (
     fake_juju_client,
     fake_juju_client_optional_jes,
-)
+    )
 from jujuconfig import (
     get_environments_path,
     get_jenv_path,
@@ -65,27 +65,27 @@
     KILL_CONTROLLER,
     SimpleEnvironment,
     Status,
-)
+    )
 from remote import (
     _Remote,
     remote_from_address,
     SSHRemote,
     winrm,
-)
+    )
 from tests import (
     FakeHomeTestCase,
     temp_os_env,
     use_context,
-)
+    )
 from tests.test_jujupy import (
     assert_juju_call,
     FakePopen,
     observable_temp_file,
-)
+    )
 from utility import (
     LoggedException,
     temp_dir,
-)
+    )
 
 
 def make_logs(log_dir):
@@ -1647,6 +1647,36 @@
         wfp_mock.assert_called_once_with(
             'bootstrap.example.org', 22, timeout=120)
 
+    @contextmanager
+    def booted_to_bootstrap(self, bs_manager):
+        """Preform patches to focus on the call to bootstrap."""
+        with patch.object(bs_manager, 'dump_all_logs'):
+            with patch.object(bs_manager, 'runtime_context'):
+                with patch.object(bs_manager.client, 'juju'):
+                    with patch.object(bs_manager.client, 'bootstrap') as mock:
+                        yield mock
+
+    def test_booted_context_kwargs(self):
+        client = fake_juju_client()
+        with temp_dir() as root:
+            log_dir = os.path.join(root, 'log-dir')
+            os.mkdir(log_dir)
+            bs_manager = BootstrapManager(
+                'foobar', client, client,
+                None, [], None, None, None, None, log_dir, False,
+                True, True)
+            juju_home = os.path.join(root, 'juju-home')
+            os.mkdir(juju_home)
+            client.env.juju_home = juju_home
+            with self.booted_to_bootstrap(bs_manager) as bootstrap_mock:
+                with bs_manager.booted_context(False, to='test'):
+                    bootstrap_mock.assert_called_once_with(
+                        upload_tools=False, to='test', bootstrap_series=None)
+            with self.booted_to_bootstrap(bs_manager) as bootstrap_mock:
+                with bs_manager.existing_booted_context(False, to='test'):
+                    bootstrap_mock.assert_called_once_with(
+                        upload_tools=False, to='test', bootstrap_series=None)
+
     def test_runtime_context_teardown_ignores_soft_deadline(self):
         env = JujuData('foo', {'type': 'nonlocal'})
         soft_deadline = datetime(2015, 1, 2, 3, 4, 6)

=== modified file 'tests/test_jujupy.py'
--- tests/test_jujupy.py	2016-09-23 15:02:11 +0000
+++ tests/test_jujupy.py	2016-09-27 15:23:51 +0000
@@ -958,7 +958,7 @@
                 '--default-model', 'foo', '--agent-version', '2.0',
                 '--credential', 'credential_name'), include_e=False)
 
-    def test_bootstrap_args(self):
+    def test_bootstrap_bootstrap_series(self):
         env = JujuData('foo', {'type': 'bar', 'region': 'baz'})
         client = EnvJujuClient(env, '2.0-zeta1', None)
         with patch.object(client, 'juju') as mock:
@@ -971,6 +971,43 @@
                 '--agent-version', '2.0',
                 '--bootstrap-series', 'angsty'), include_e=False)
 
+    def test_bootstrap_auto_upgade(self):
+        env = JujuData('foo', {'type': 'bar', 'region': 'baz'})
+        client = EnvJujuClient(env, '2.0-zeta1', None)
+        with patch.object(client, 'juju') as mock:
+            with observable_temp_file() as config_file:
+                client.bootstrap(auto_upgrade=True)
+        mock.assert_called_with(
+            'bootstrap', (
+                '--constraints', 'mem=2G', 'foo', 'bar/baz',
+                '--config', config_file.name, '--default-model', 'foo',
+                '--agent-version', '2.0', '--auto-upgrade'), include_e=False)
+
+    def test_bootstrap_metadata(self):
+        env = JujuData('foo', {'type': 'bar', 'region': 'baz'})
+        client = EnvJujuClient(env, '2.0-zeta1', None)
+        with patch.object(client, 'juju') as mock:
+            with observable_temp_file() as config_file:
+                client.bootstrap(metadata_source='/var/test-source')
+        mock.assert_called_with(
+            'bootstrap', (
+                '--constraints', 'mem=2G', 'foo', 'bar/baz',
+                '--config', config_file.name, '--default-model', 'foo',
+                '--agent-version', '2.0',
+                '--metadata-source', '/var/test-source'), include_e=False)
+
+    def test_bootstrap_to(self):
+        env = JujuData('foo', {'type': 'bar', 'region': 'baz'})
+        client = EnvJujuClient(env, '2.0-zeta1', None)
+        with patch.object(client, 'juju') as mock:
+            with observable_temp_file() as config_file:
+                client.bootstrap(to='target')
+        mock.assert_called_with(
+            'bootstrap', (
+                '--constraints', 'mem=2G', 'foo', 'bar/baz',
+                '--config', config_file.name, '--default-model', 'foo',
+                '--agent-version', '2.0', '--to', 'target'), include_e=False)
+
     def test_bootstrap_async(self):
         env = JujuData('foo', {'type': 'bar', 'region': 'baz'})
         with patch.object(EnvJujuClient, 'juju_async', autospec=True) as mock:

